/*
 * A resource offering support for creating contexts and opening files.
 *
 * Copyright (C) 2021, 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include "opener_server.h"
#include "opener_resource.h"
#include "resource_server.h"

/* Support for providing access to files. */

OpenerResource::OpenerResource(ResourceRegistry *registry)
: _registry(registry)
{
}

OpenerResource::~OpenerResource()
{
}

ipc_server_default_config_type OpenerResource::config()
{
    return config_Opener;
}



/* Obtain any active parent directory for a path, notifying its subscribers of
   the file opening event.
   NOTE: This might need to happen in the registry due to concurrency issues. */

long OpenerResource::notify_parent(const char *path)
{
    Provider *provider;
    fileid_t fileid;
    long err;

    err = get_parent(path, &fileid);

    if (err)
        return err;

    /* Obtain the provider of the parent directory. */

    err = _registry->find_provider(fileid, &provider);

    if (err)
        return err;

    /* With any active provider, send a notification.
       NOTE: This should also communicate which file was involved, probably
       NOTE: using the file identifier of the opened file. */

    provider->notify_all(NOTIFY_FILE_OPENED);
    return L4_EOK;
}



/* Make a directory. */

long OpenerResource::mkdir(const char *path, sys_mode_t mode)
{
    return make_directory_object(path, mode);
}

/* Return an object for the given path and flags. */

long OpenerResource::open(const char *path, flags_t flags, offset_t *size,
                          l4_cap_idx_t *cap, object_flags_t *object_flags)
{
    Resource *resource;
    long err = _registry->get_resource(this, path, flags, size, object_flags, &resource);

    if (err)
        return err;

    /* Notify listeners subscribed to the parent of the opened object. */

    notify_parent(path);

    /* Start a server for the resource. */

    return ResourceServer(resource).start_thread(cap);
}

/* Request the removal of the named filesystem object. */

long OpenerResource::remove(const char *path)
{
    return _registry->remove_provider(this, path);
}

/* Request the renaming of the named filesystem object. */

long OpenerResource::rename(const char *source, const char *target)
{
    return _registry->rename_provider(this, source, target);
}

/* Obtain statistics metadata for the named filesystem object. */

long OpenerResource::stat(const char *path, void *base, offset_t size)
{
    return stat_object(path, base, size);
}



/* Opener interface methods. */

long OpenerResource::context(l4_cap_idx_t *context)
{
    OpenerContextResource *resource = new OpenerContextResource(this);

    /* Complete the initialisation and start a server in a new thread.
       If the thread does not start, the resource will be finalised. */

    return ResourceServer(resource).start_thread(context);
}

// vim: tabstop=4 expandtab shiftwidth=4
